@@ -5,12 +5,15 @@
 #include "../include/sane/sanei.h"
 #include "../include/sane/saneopts.h"
 #include <sys/types.h>
+#ifdef HAVE_SYS_IPC_H
 #include <sys/ipc.h>
+#endif
 #include <unistd.h>
 #include <fcntl.h>
 #include <errno.h>
 
 #include "../include/sane/sanei_usb.h"
+#include "../include/sane/sanei_thread.h"
 
 #define _MAX_ID_LEN 20
 
@@ -172,6 +175,8 @@ struct Artec48U_Device
   size_t read_pos;
   size_t read_bytes_in_buffer;
   size_t read_bytes_left;
+  unsigned int is_epro;
+  unsigned int epro_mult;
 };
 
 /** Scan parameters for artec48u_device_setup_scan().
@@ -259,6 +264,16 @@ struct Artec48U_Line_Reader
 			 unsigned int **buffer_pointers_return);
 };
 
+#ifndef SANE_OPTION
+typedef union
+{
+  SANE_Word w;
+  SANE_Word *wa;		/* word array */
+  SANE_String s;
+}
+Option_Value;
+#endif
+
 struct Artec48U_Scanner
 {
   Artec48U_Scanner *next;
@@ -267,8 +282,9 @@ struct Artec48U_Scanner
   Artec48U_Device *dev;
   Artec48U_Line_Reader *reader;
   FILE *pipe_handle;
-  int reader_pid;
+  SANE_Pid reader_pid;
   int pipe;
+  int reader_pipe;
   SANE_Option_Descriptor opt[NUM_OPTIONS];
   Option_Value val[NUM_OPTIONS];
   SANE_Status exit_code;
@@ -282,7 +298,7 @@ struct Artec48U_Scanner
   SANE_Byte *line_buffer;
   SANE_Byte *lineart_buffer;
   SANE_Word lines_to_read;
-  unsigned int temp_shading_buffer[3][5120];
+  unsigned int temp_shading_buffer[3][10240]; /*epro*/
   unsigned int *buffer_pointers[3];
   unsigned char *shading_buffer_w;
   unsigned char *shading_buffer_b;
