@@ -3,7 +3,7 @@
 
    dc25.c 
 
-   $Id: dc25.c,v 1.16 2004/10/03 17:34:33 hmg-guest Exp $
+   $Id: dc25.c,v 1.18 2008-11-26 21:21:24 kitno-guest Exp $
 
    This file (C) 1998 Peter Fales
 
@@ -1845,12 +1845,12 @@ sane_init (SANE_Int * version_code, SANE_Auth_Callback UNUSEDARG authorize)
   DBG_INIT ();
 
   if (version_code)
-    *version_code = SANE_VERSION_CODE (V_MAJOR, V_MINOR, 0);
+    *version_code = SANE_VERSION_CODE (SANE_CURRENT_MAJOR, V_MINOR, 0);
 
   fp = sanei_config_open (DC25_CONFIG_FILE);
 
   DBG (127,
-       "sane_init() $Id: dc25.c,v 1.16 2004/10/03 17:34:33 hmg-guest Exp $\n");
+       "sane_init() $Id: dc25.c,v 1.18 2008-11-26 21:21:24 kitno-guest Exp $\n");
 
   if (!fp)
     {
@@ -1907,7 +1907,7 @@ sane_init (SANE_Int * version_code, SANE_Auth_Callback UNUSEDARG authorize)
 		  tty_baud = DEFAULT_TTY_BAUD;
 		  break;
 		}
-	      DBG (20, "Config file baud=%d\n", tty_baud);
+	      DBG (20, "Config file baud=%lu\n", (u_long) tty_baud);
 	    }
 	  else if (strcmp (dev_name, "dumpinquiry") == 0)
 	    {
