@@ -15,7 +15,7 @@ enum fujitsu_Option
 {
   OPT_NUM_OPTS = 0,
 
-  OPT_MODE_GROUP,
+  OPT_STANDARD_GROUP,
   OPT_SOURCE, /*fb/adf/front/back/duplex*/
   OPT_MODE,   /*mono/gray/color*/
   OPT_X_RES,  /*a range or a list*/
@@ -31,17 +31,64 @@ enum fujitsu_Option
 
   OPT_ENHANCEMENT_GROUP,
   OPT_BRIGHTNESS,
-  OPT_THRESHOLD,
   OPT_CONTRAST,
+  OPT_GAMMA,
+  OPT_THRESHOLD,
+
+  /*IPC*/
   OPT_RIF,
+  OPT_HT_TYPE,
+  OPT_HT_PATTERN,
+  OPT_OUTLINE,
+  OPT_EMPHASIS,
+  OPT_SEPARATION,
+  OPT_MIRRORING,
+  OPT_WL_FOLLOW,
+  OPT_IPC_MODE,
+
+  /*IPC/DTC*/
+  OPT_BP_FILTER,
+  OPT_SMOOTHING,
+  OPT_GAMMA_CURVE,
+  OPT_THRESHOLD_CURVE,
+  OPT_THRESHOLD_WHITE,
+  OPT_NOISE_REMOVAL,
+  OPT_MATRIX_5,
+  OPT_MATRIX_4,
+  OPT_MATRIX_3,
+  OPT_MATRIX_2,
+
+  /*IPC/SDTC*/
+  OPT_VARIANCE,
 
   OPT_ADVANCED_GROUP,
+  OPT_COMPRESS,
+  OPT_COMPRESS_ARG,
+  OPT_DF_ACTION,
+  OPT_DF_SKEW,
+  OPT_DF_THICKNESS,
+  OPT_DF_LENGTH,
+  OPT_DF_DIFF,
+  OPT_BG_COLOR,
   OPT_DROPOUT_COLOR,
+  OPT_BUFF_MODE,
+  OPT_PREPICK,
+  OPT_OVERSCAN,
   OPT_SLEEP_TIME,
   OPT_DUPLEX_OFFSET,
   OPT_GREEN_OFFSET,
   OPT_BLUE_OFFSET,
-  OPT_USE_SWAPFILE,
+
+  OPT_ENDORSER_GROUP,
+  OPT_ENDORSER,
+  OPT_ENDORSER_BITS,
+  OPT_ENDORSER_VAL,
+  OPT_ENDORSER_STEP,
+  OPT_ENDORSER_Y,
+  OPT_ENDORSER_FONT,
+  OPT_ENDORSER_DIR,
+  OPT_ENDORSER_SIDE,
+  OPT_ENDORSER_STRING,
 
   OPT_SENSOR_GROUP,
   OPT_TOP,
@@ -74,7 +121,8 @@ struct fujitsu
   /* --------------------------------------------------------------------- */
   /* immutable values which are set during init of scanner.                */
   struct fujitsu *next;
-  char *device_name;            /* The name of the scanner device for sane */
+  char device_name[1024];             /* The name of the device from sanei */
+  int missing; 				/* used to mark unplugged scanners */
 
   /* --------------------------------------------------------------------- */
   /* immutable values which are set during reading of config file.         */
@@ -84,14 +132,13 @@ struct fujitsu
   /* --------------------------------------------------------------------- */
   /* immutable values which are set during inquiry probing of the scanner. */
   /* members in order found in scsi data...                                */
-  SANE_Device sane;
-
   char vendor_name[9];          /* raw data as returned by SCSI inquiry.   */
-  char product_name[17];        /* raw data as returned by SCSI inquiry.   */
+  char model_name[17];          /* raw data as returned by SCSI inquiry.   */
   char version_name[5];         /* raw data as returned by SCSI inquiry.   */
 
   int color_raster_offset;      /* offset between r and b scan line and    */
                                 /* between b and g scan line (0 or 4)      */
+
   int duplex_raster_offset;     /* offset between front and rear page when */
                                 /* when scanning 3091 style duplex         */
 
@@ -132,25 +179,53 @@ struct fujitsu
   int can_monochrome;
   int can_halftone;
   int can_grayscale;
-  int can_color_monochrome;
-  int can_color_halftone;
   int can_color_grayscale;
 
   /* --------------------------------------------------------------------- */
   /* immutable values which are set during vndr VPD probing of the scanner */
   /* members in order found in scsi data...                                */
-  int has_operator_panel;
-  int has_barcode;
-  int has_imprinter;
-  int has_duplex;
-  int has_transparency;
-  int has_flatbed;
   int has_adf;
+  int has_flatbed;
+  int has_transparency;
+  int has_duplex;
+  int has_endorser_b;
+  int has_barcode;
+  int has_operator_panel;
+  int has_endorser_f;
 
   int adbits;
   int buffer_bytes;
 
-  /*FIXME: do we need the std cmd list? */
+  /*supported scsi commands*/
+  int has_cmd_msen10;
+  int has_cmd_msel10;
+
+  int has_cmd_lsen;
+  int has_cmd_lsel;
+  int has_cmd_change;
+  int has_cmd_rbuff;
+  int has_cmd_wbuff;
+  int has_cmd_cav;
+  int has_cmd_comp;
+  int has_cmd_gdbs;
+
+  int has_cmd_op;
+  int has_cmd_send;
+  int has_cmd_read;
+  int has_cmd_gwin;
+  int has_cmd_swin;
+  int has_cmd_sdiag;
+  int has_cmd_rdiag;
+  int has_cmd_scan;
+  
+  int has_cmd_msen6;
+  int has_cmd_copy;
+  int has_cmd_rel;
+  int has_cmd_runit;
+  int has_cmd_msel6;
+  int has_cmd_inq;
+  int has_cmd_rs;
+  int has_cmd_tur;
 
   /*FIXME: there are more vendor cmds? */
   int has_cmd_subwindow;
@@ -170,14 +245,17 @@ struct fujitsu
   int num_download_dither;
 
   int has_rif;
-  int has_auto1;
-  int has_auto2;
+  int has_dtc;
+  int has_sdtc;
   int has_outline;
   int has_emphasis;
   int has_autosep;
   int has_mirroring;
-  int has_white_level_follow;
+  int has_wl_follow;
   int has_subwindow;
+  int has_diffusion;
+  int has_ipc3;
+  int has_rotation;
 
   int has_comp_MH;
   int has_comp_MR;
@@ -187,10 +265,32 @@ struct fujitsu
   int has_comp_JPG2;
   int has_comp_JPG3;
 
-  /*FIXME: endorser data? */
+  /*FIXME: more endorser data? */
+  int endorser_type_f;
+  int endorser_type_b;
 
   /*FIXME: barcode data? */
 
+  /* overscan size in pixels comes from scanner in basic res units */
+  int os_x_basic;
+  int os_y_basic;
+
+  /* --------------------------------------------------------------------- */
+  /* immutable values which are gathered by mode_sense command     */
+
+  int has_MS_unknown;
+  int has_MS_prepick;
+  int has_MS_sleep;
+  int has_MS_duplex;
+  int has_MS_rand;
+  int has_MS_bg;
+  int has_MS_df;
+  int has_MS_dropout; /* dropout color specified in mode select data */
+  int has_MS_buff;
+  int has_MS_auto;
+  int has_MS_lamp;
+  int has_MS_jobsep;
+
   /* --------------------------------------------------------------------- */
   /* immutable values which are hard coded because they are not in vpd     */
   /* this section replaces all the old 'switch (s->model)' code            */
@@ -200,21 +300,40 @@ struct fujitsu
   int max_y;
   int min_x;
   int min_y;
+  int max_x_fb;
+  int max_y_fb;
 
-  int has_back;       /* not all duplex scanners can do adf back side only */
-  int color_interlace;  /* different models interlace colors differently   */
-  int duplex_interlace; /* different models interlace sides differently    */
-  int has_MS_dropout; /* dropout color specified in mode select data */
-  int has_SW_dropout; /* dropout color specified in set window data */
-  int window_vid; /* some models want different vendor ID in set window */
+  int has_back;         /* not all duplex scanners can do adf back side only */
+  int color_interlace;  /* different models interlace colors differently     */
+  int duplex_interlace; /* different models interlace sides differently      */
+  int jpeg_interlace;   /* different models interlace jpeg sides differently */
+  int cropping_mode;    /* lower-end scanners dont crop from paper size      */
   int ghs_in_rs;
+  int window_gamma;
+  int endorser_string_len;
+
+  int broken_diag_serial;   /* some scanners are just plain borked */
+
+  int has_vuid_mono;    /* mono set window data */
+  int has_vuid_3091;    /* 3091/2 set window data */
+  int has_vuid_color;   /* color set window data */
 
   int reverse_by_mode[6]; /* mode specific */
+  int ppl_mod_by_mode[6]; /* mode specific scanline length limitation */
+
+  /* --------------------------------------------------------------------- */
+  /* immutable values which are set during serial number probing scanner   */
+  char serial_name[28];        /* 16 char model, ':', 10 byte serial, null */
+
+  /* --------------------------------------------------------------------- */
+  /* struct with pointers to device/vendor/model names, and a type value */
+  /* used to inform sane frontend about the device */
+  SANE_Device sane;
+
   /* --------------------------------------------------------------------- */
   /* changeable SANE_Option structs provide our interface to frontend.     */
   /* some options require lists of strings or numbers, we keep them here   */
   /* instead of in global vars so that they can differ for each scanner    */
-  /* these are loaded in init_options(), but may be changed by other funcs */
 
   /* long array of option structs */
   SANE_Option_Descriptor opt[NUM_OPTIONS];
@@ -238,19 +357,45 @@ struct fujitsu
 
   /*enhancement group*/
   SANE_Range brightness_range;
-  SANE_Range threshold_range;
   SANE_Range contrast_range;
+  SANE_Range gamma_range;
+  SANE_Range threshold_range;
 
   /*ipc group*/
+  SANE_String_Const ht_type_list[4];
+  SANE_Range ht_pattern_range;
+  SANE_Range emphasis_range;
+  SANE_String_Const wl_follow_list[4];
+  SANE_String_Const ipc_mode_list[4];
+  SANE_Range gamma_curve_range;
+  SANE_Range threshold_curve_range;
+  SANE_Range variance_range;
 
   /*advanced group*/
+  SANE_String_Const compress_list[3];
+  SANE_Range compress_arg_range;
+  SANE_String_Const df_action_list[4];
+  SANE_String_Const df_diff_list[5];
+  SANE_String_Const bg_color_list[4];
   SANE_String_Const do_color_list[5];
   SANE_String_Const lamp_color_list[5];
+  SANE_String_Const buff_mode_list[4];
+  SANE_String_Const prepick_list[4];
+  SANE_String_Const overscan_list[4];
   SANE_Range sleep_time_range;
   SANE_Range duplex_offset_range;
   SANE_Range green_offset_range;
   SANE_Range blue_offset_range;
 
+  /*endorser group*/
+  SANE_Range endorser_bits_range;
+  SANE_Range endorser_val_range;
+  SANE_Range endorser_step_range;
+  SANE_Range endorser_y_range;
+  SANE_String_Const endorser_font_list[6];
+  SANE_String_Const endorser_dir_list[3];
+  SANE_String_Const endorser_side_list[3];
+
   /* --------------------------------------------------------------------- */
   /* changeable vars to hold user input. modified by SANE_Options above    */
 
@@ -271,15 +416,49 @@ struct fujitsu
 
   /*enhancement group*/
   int brightness;
-  int threshold;
   int contrast;
-  int rif;
+  int gamma;
+  int threshold;
 
-  /*ipc group*/
-  int gamma; /* not currently user settable */
+  /* ipc */
+  int rif;
+  int ht_type;
+  int ht_pattern;
+  int outline;
+  int emphasis;
+  int separation;
+  int mirroring;
+  int wl_follow;
+  int ipc_mode;
+
+  /* ipc_mode=DTC */
+  int bp_filter;
+  int smoothing;
+  int gamma_curve;
+  int threshold_curve;
+  int threshold_white;
+  int noise_removal;
+  int matrix_5;
+  int matrix_4;
+  int matrix_3;
+  int matrix_2;
+
+  /* ipc_mode = SDTC */
+  int variance;
 
   /*advanced group*/
+  int compress;
+  int compress_arg;
+  int df_action;
+  int df_skew;
+  int df_thickness;
+  int df_length;
+  int df_diff;
+  int bg_color;
   int dropout_color;
+  int buff_mode;
+  int prepick;
+  int overscan;
   int lamp_color;
   int sleep_time;
   int duplex_offset;
@@ -287,6 +466,17 @@ struct fujitsu
   int blue_offset;
   int use_temp_file;
 
+  /*endorser group*/
+  int u_endorser;
+  int u_endorser_bits;
+  int u_endorser_val;
+  int u_endorser_step;
+  int u_endorser_y;
+  int u_endorser_font;
+  int u_endorser_dir;
+  int u_endorser_side;
+  char u_endorser_string[81]; /*max length, plus null byte*/
+
   /* --------------------------------------------------------------------- */
   /* values which are derived from setting the options above */
   /* the user never directly modifies these */
@@ -304,7 +494,9 @@ struct fujitsu
   /* --------------------------------------------------------------------- */
   /* values which are set by scanning functions to keep track of pages, etc */
   int started;
-  int img_count; /* how many 'sides' delivered */
+  int reading;
+  int cancelled;
+  int side;
 
   /* total to read/write */
   int bytes_tot[2];
@@ -317,16 +509,22 @@ struct fujitsu
   int bytes_tx[2];
 
   unsigned char * buffers[2];
-  int fds[2];
+
+  /* --------------------------------------------------------------------- */
+  /* values used by the compression functions, esp. jpeg with duplex       */
+  int jpeg_stage;
+  int jpeg_ff_offset;
+  int jpeg_front_rst;
+  int jpeg_back_rst;
+  int jpeg_x_byte;
 
   /* --------------------------------------------------------------------- */
   /* values which used by the command and data sending functions (scsi/usb)*/
   int fd;                      /* The scanner device file descriptor.     */
-  unsigned char rs_buffer[RS_return_size];
+  size_t rs_info;
 
   /* --------------------------------------------------------------------- */
   /* values which are used by the get hardware status command              */
-  time_t last_ghs;
 
   int hw_top;
   int hw_A3;
@@ -354,6 +552,9 @@ struct fujitsu
 
   int hw_duplex_sw;
   int hw_density_sw;
+
+  /* values which are used to track the frontend's access to sensors  */
+  char hw_read[NUM_OPTIONS-OPT_TOP];
 };
 
 #define CONNECTION_SCSI   0 /* SCSI interface */
@@ -367,6 +568,16 @@ struct fujitsu
 #define SOURCE_ADF_BACK 2
 #define SOURCE_ADF_DUPLEX 3
 
+#define COMP_NONE WD_cmp_NONE
+#define COMP_JPEG WD_cmp_JPG1
+
+#define JPEG_STAGE_HEAD 0
+#define JPEG_STAGE_SOF 1
+#define JPEG_STAGE_SOS 2
+#define JPEG_STAGE_FRONT 3
+#define JPEG_STAGE_BACK 4
+#define JPEG_STAGE_EOI 5
+
 /* these are same as scsi data to make code easier */
 #define MODE_LINEART WD_comp_LA
 #define MODE_HALFTONE WD_comp_HT
@@ -375,28 +586,56 @@ struct fujitsu
 #define MODE_COLOR_HALFTONE WD_comp_CH
 #define MODE_COLOR WD_comp_CG
 
-/* these are same as scsi data to make code easier */
-#define COLOR_DEFAULT MSEL_dropout_DEFAULT
-#define COLOR_RED MSEL_dropout_RED
-#define COLOR_GREEN MSEL_dropout_GREEN
-#define COLOR_BLUE MSEL_dropout_BLUE
+/* these are same as dropout scsi data to make code easier */
+#define COLOR_DEFAULT 0
+#define COLOR_GREEN 8
+#define COLOR_RED 9
+#define COLOR_BLUE 11
 
-#define COLOR_INTERLACE_NONE 0
-#define COLOR_INTERLACE_3091 1
+#define COLOR_WHITE 1
+#define COLOR_BLACK 2
+
+#define COLOR_INTERLACE_UNK 0
+#define COLOR_INTERLACE_RGB 1
 #define COLOR_INTERLACE_BGR 2
 #define COLOR_INTERLACE_RRGGBB 3
+#define COLOR_INTERLACE_3091 4
 
-#define DUPLEX_INTERLACE_NONE 0 
-#define DUPLEX_INTERLACE_3091 1 
+#define DUPLEX_INTERLACE_ALT 0 
+#define DUPLEX_INTERLACE_NONE 1 
+#define DUPLEX_INTERLACE_3091 2 
+
+#define JPEG_INTERLACE_ALT 0 
+#define JPEG_INTERLACE_NONE 1 
+
+#define CROP_RELATIVE 0 
+#define CROP_ABSOLUTE 1 
+
+#define DF_DEFAULT 0
+#define DF_CONTINUE 1
+#define DF_STOP 2
+
+#define FONT_H  0
+#define FONT_HB 1
+#define FONT_HN 2
+#define FONT_V  3
+#define FONT_VB 4
+
+#define DIR_TTB 0
+#define DIR_BTT 1
+
+/* endorser type, same as scsi inquiry data */
+#define ET_OLD	0
+#define ET_30	1
+#define ET_40	2
 
-#define GHS_TIME 0 /* seconds passed before calling GHS */
 /* ------------------------------------------------------------------------- */
 
 #define MM_PER_INCH    25.4
 #define MM_PER_UNIT_UNFIX SANE_UNFIX(SANE_FIX(MM_PER_INCH / 1200.0))
 #define MM_PER_UNIT_FIX SANE_FIX(SANE_UNFIX(SANE_FIX(MM_PER_INCH / 1200.0)))
 
-#define SCANNER_UNIT_TO_FIXED_MM(number) SANE_FIX(number * MM_PER_UNIT_UNFIX)
+#define SCANNER_UNIT_TO_FIXED_MM(number) SANE_FIX((number) * MM_PER_UNIT_UNFIX)
 #define FIXED_MM_TO_SCANNER_UNIT(number) SANE_UNFIX(number) / MM_PER_UNIT_UNFIX
 
 #define FUJITSU_CONFIG_FILE "fujitsu.conf"
@@ -449,8 +688,6 @@ void sane_exit (void);
 
 /* ------------------------------------------------------------------------- */
 
-static SANE_Status find_scanners (void);
-
 static SANE_Status attach_one_scsi (const char *name);
 static SANE_Status attach_one_usb (const char *name);
 static SANE_Status attach_one (const char *devicename, int connType);
@@ -462,48 +699,69 @@ static SANE_Status sense_handler (int scsi_fd, u_char * result, void *arg);
 
 static SANE_Status init_inquire (struct fujitsu *s);
 static SANE_Status init_vpd (struct fujitsu *s);
+static SANE_Status init_ms (struct fujitsu *s);
 static SANE_Status init_model (struct fujitsu *s);
+static SANE_Status init_user (struct fujitsu *s);
 static SANE_Status init_options (struct fujitsu *scanner);
+static SANE_Status init_interlace (struct fujitsu *scanner);
+static SANE_Status init_serial (struct fujitsu *scanner);
 
 static SANE_Status
 do_cmd(struct fujitsu *s, int runRS, int shortTime,
  unsigned char * cmdBuff, size_t cmdLen,
  unsigned char * outBuff, size_t outLen,
- unsigned char * inBuff, size_t inLen
+ unsigned char * inBuff, size_t * inLen
 );
 
 static SANE_Status
 do_scsi_cmd(struct fujitsu *s, int runRS, int shortTime,
  unsigned char * cmdBuff, size_t cmdLen,
  unsigned char * outBuff, size_t outLen,
- unsigned char * inBuff, size_t inLen
+ unsigned char * inBuff, size_t * inLen
 );
 
 static SANE_Status
 do_usb_cmd(struct fujitsu *s, int runRS, int shortTime,
  unsigned char * cmdBuff, size_t cmdLen,
  unsigned char * outBuff, size_t outLen,
- unsigned char * inBuff, size_t inLen
+ unsigned char * inBuff, size_t * inLen
 );
 
-static int wait_scanner (struct fujitsu *s);
+static SANE_Status wait_scanner (struct fujitsu *s);
 
-static int object_position (struct fujitsu *s, int i_load);
-
-static SANE_Status do_cancel (struct fujitsu *scanner);
+static SANE_Status object_position (struct fujitsu *s, int i_load);
 
 static SANE_Status scanner_control (struct fujitsu *s, int function);
+static SANE_Status scanner_control_ric (struct fujitsu *s, int bytes, int side);
+
+static SANE_Status mode_select_df(struct fujitsu *s);
 
 static SANE_Status mode_select_dropout(struct fujitsu *s);
 
+static SANE_Status mode_select_bg(struct fujitsu *s);
+
+static SANE_Status mode_select_buff (struct fujitsu *s);
+
+static SANE_Status mode_select_prepick (struct fujitsu *s);
+
+static SANE_Status mode_select_overscan (struct fujitsu *s);
+
 static SANE_Status set_sleep_mode(struct fujitsu *s);
 
-int get_current_side (struct fujitsu *s);
+static int get_page_width (struct fujitsu *s);
+static int get_page_height (struct fujitsu *s);
 
-static int set_window (struct fujitsu *s);
+static SANE_Status send_lut (struct fujitsu *s);
+static SANE_Status send_endorser (struct fujitsu *s);
+static SANE_Status endorser (struct fujitsu *s);
+static SANE_Status set_window (struct fujitsu *s);
+static SANE_Status get_pixelsize(struct fujitsu *s);
 
-static int start_scan (struct fujitsu *s);
+static SANE_Status start_scan (struct fujitsu *s);
 
+static SANE_Status check_for_cancel(struct fujitsu *s);
+
+static SANE_Status read_from_JPEGduplex(struct fujitsu *s);
 static SANE_Status read_from_3091duplex(struct fujitsu *s);
 static SANE_Status read_from_scanner(struct fujitsu *s, int side);
 
@@ -512,11 +770,9 @@ static SANE_Status copy_buffer(struct fujitsu *s, unsigned char * buf, int len,
 
 static SANE_Status read_from_buffer(struct fujitsu *s, SANE_Byte * buf, SANE_Int max_len, SANE_Int * len, int side);
 
-static void calculateDerivedValues (struct fujitsu *scanner);
-
 static SANE_Status setup_buffers (struct fujitsu *s);
 
-static SANE_Status get_hardware_status (struct fujitsu *s);
+static SANE_Status get_hardware_status (struct fujitsu *s, SANE_Int option);
 
 static void hexdump (int level, char *comment, unsigned char *p, int l);
 
