@@ -663,7 +663,7 @@ sane_init (SANE_Int * version_code, SANE_Auth_Callback UNUSEDARG authorize)
   DBG_INIT ();
 
   DBG (1,
-       "GPHOTO2 Backend $Id: gphoto2.c,v 1.32 2005/12/03 18:23:55 psfales-guest Exp $\n");
+       "GPHOTO2 Backend $Id: gphoto2.c,v 1.34 2009-04-30 13:58:35 kitno-guest Exp $\n");
 
   if (getenv ("GP_DEBUG"))
     {
@@ -671,7 +671,7 @@ sane_init (SANE_Int * version_code, SANE_Auth_Callback UNUSEDARG authorize)
     }
 
   if (version_code)
-    *version_code = SANE_VERSION_CODE (V_MAJOR, V_MINOR, 0);
+    *version_code = SANE_VERSION_CODE (SANE_CURRENT_MAJOR, V_MINOR, 0);
 
   fp = sanei_config_open (GPHOTO2_CONFIG_FILE);
 
@@ -728,13 +728,21 @@ sane_init (SANE_Int * version_code, SANE_Auth_Callback UNUSEDARG authorize)
 		}
 	      for (n = 0; n < entries; n++)
 		{
+#ifdef HAVE_GP_PORT_INFO_GET_PATH
+		  char *info_path = NULL;
+#endif
 		  result = gp_port_info_list_get_info (list, n, &info);
 		  if (result < 0)
 		    {
 		      gp_port_info_list_free (list);
 		      return SANE_STATUS_INVAL;
 		    }
+#ifdef HAVE_GP_PORT_INFO_GET_PATH
+		  gp_port_info_get_path (info, &info_path);
+		  if (strcmp (Cam_data.port, info_path) == 0)
+#else
 		  if (strcmp (Cam_data.port, info.path) == 0)
+#endif
 		    {
 		      break;
 		    }
