@@ -481,7 +481,7 @@ sane_init (SANE_Int * piVersion, SANE_Auth_Callback pfnAuth)
   DBG_INIT ();
 
   HP5400_DBG (DBG_MSG, "sane_init: SANE hp5400 backend version %d.%d-%d (from %s)\n",
-       V_MAJOR, V_MINOR, BUILD, PACKAGE_STRING);
+       SANE_CURRENT_MAJOR, V_MINOR, BUILD, PACKAGE_STRING);
 
   sanei_usb_init ();
 
@@ -529,7 +529,7 @@ sane_init (SANE_Int * piVersion, SANE_Auth_Callback pfnAuth)
 
   if (piVersion != NULL)
     {
-      *piVersion = SANE_VERSION_CODE (V_MAJOR, V_MINOR, BUILD);
+      *piVersion = SANE_VERSION_CODE (SANE_CURRENT_MAJOR, V_MINOR, BUILD);
     }
 
 
