@@ -1,6 +1,6 @@
 /* sane - Scanner Access Now Easy.
 
-   Copyright (C) 2002 Frank Zago (fzago@greshamstorage.com)
+   Copyright (C) 2002 Frank Zago (sane at zago dot net)
 
    This file is part of the SANE package.
 
@@ -42,7 +42,7 @@
 */
 
 /* 
-	$Id: leo.h,v 1.1 2002/04/21 23:12:02 fzago Exp $
+	$Id: leo.h,v 1.4 2005-07-07 11:55:42 fzago-guest Exp $
 */
 
 /* Commands supported by the scanner. */
@@ -240,14 +240,6 @@ enum Leo_Option
   OPT_NUM_OPTIONS
 };
 
-typedef union
-{
-  SANE_Word w;			/* word */
-  SANE_Word *wa;		/* word array */
-  SANE_String s;		/* string */
-}
-Option_Value;
-
 /*--------------------------------------------------------------------------*/
 
 /* 
