@@ -3,7 +3,7 @@
    Copyright (C) 2000 Mustek.
    Originally maintained by Tom Wang <tom.wang@mustek.com.tw>
 
-   Copyright (C) 2001, 2002 by Henning Meier-Geinitz.
+   Copyright (C) 2001 - 2004 by Henning Meier-Geinitz.
 
    This file is part of the SANE package.
 
@@ -46,7 +46,7 @@
    This file implements a SANE backend for Mustek 1200UB and similar 
    USB flatbed scanners.  */
 
-#define BUILD 17
+#define BUILD 18
 
 #include "../include/sane/config.h"
 
@@ -86,8 +86,8 @@ static Mustek_Usb_Device *first_dev;
 static Mustek_Usb_Scanner *first_handle;
 static const SANE_Device **devlist = 0;
 
-/* Maximum amount of data read in one turn from USB */
-static SANE_Word max_block_size = 8 * 1024;
+/* Maximum amount of data read in one turn from USB. */
+static SANE_Word max_block_size = (8 * 1024);
 
 /* Array of newly attached devices */
 static Mustek_Usb_Device **new_dev;
@@ -704,11 +704,11 @@ sane_init (SANE_Int * version_code, SANE_Auth_Callback authorize)
   FILE *fp;
 
   DBG_INIT ();
-  DBG (2, "SANE Mustek USB backend version %d.%d build %d from %s\n", V_MAJOR,
+  DBG (2, "SANE Mustek USB backend version %d.%d build %d from %s\n", SANE_CURRENT_MAJOR,
        V_MINOR, BUILD, PACKAGE_STRING);
 
   if (version_code)
-    *version_code = SANE_VERSION_CODE (V_MAJOR, V_MINOR, BUILD);
+    *version_code = SANE_VERSION_CODE (SANE_CURRENT_MAJOR, V_MINOR, BUILD);
 
   DBG (5, "sane_init: authorize %s null\n", authorize ? "!=" : "==");
 
