@@ -2021,12 +2021,12 @@ sane_init (SANE_Int * version_code, SANE_Auth_Callback authorize)
   DBG_INIT ();
   DBG (DBG_FUNC, "sane_init: start\n");
   DBG (DBG_ERR, "SANE Mustek USB2 backend version %d.%d build %d from %s\n",
-       V_MAJOR, V_MINOR, BUILD, PACKAGE_STRING);
+       SANE_CURRENT_MAJOR, V_MINOR, BUILD, PACKAGE_STRING);
 
   num_devices = 1;		/* HOLD: only one device in this backend */
 
   if (version_code != NULL)
-    *version_code = SANE_VERSION_CODE (V_MAJOR, V_MINOR, BUILD);
+    *version_code = SANE_VERSION_CODE (SANE_CURRENT_MAJOR, V_MINOR, BUILD);
 
   DBG (DBG_INFO, "sane_init: authorize %s null\n", authorize ? "!=" : "==");
 
