@@ -1,7 +1,7 @@
 /* sane - Scanner Access Now Easy.
    Copyright (C) 1997 David Mosberger-Tang
-   Copyright (C) 2003 Julien BLACHE <jb@jblache.org>
-      AF-independent code + IPv6
+   Copyright (C) 2003, 2008 Julien BLACHE <jb@jblache.org>
+      AF-independent code + IPv6, Avahi support
 
    This file is part of the SANE package.
 
@@ -66,6 +66,21 @@
 #include <netinet/in.h>
 #include <netdb.h> /* OS/2 needs this _after_ <netinet/in.h>, grrr... */
 
+#ifdef WITH_AVAHI
+# include <avahi-client/client.h>
+# include <avahi-client/lookup.h>
+
+# include <avahi-common/thread-watch.h>
+# include <avahi-common/malloc.h>
+# include <avahi-common/error.h>
+
+# define SANED_SERVICE_DNS "_sane-port._tcp"
+
+static AvahiClient *avahi_client = NULL;
+static AvahiThreadedPoll *avahi_thread = NULL;
+static AvahiServiceBrowser *avahi_browser = NULL;
+#endif /* WITH_AVAHI */
+
 #include "../include/sane/sane.h"
 #include "../include/sane/sanei.h"
 #include "../include/sane/sanei_net.h"
@@ -89,13 +104,13 @@
 #if defined (HAVE_GETADDRINFO) && defined (HAVE_GETNAMEINFO)
 # define NET_USES_AF_INDEP
 # ifdef ENABLE_IPV6
-#  define NET_VERSION "1.0.13 (AF-indep+IPv6)"
+#  define NET_VERSION "1.0.14 (AF-indep+IPv6)"
 # else
-#  define NET_VERSION "1.0.13 (AF-indep)"
+#  define NET_VERSION "1.0.14 (AF-indep)"
 # endif /* ENABLE_IPV6 */
 #else
 # undef ENABLE_IPV6
-# define NET_VERSION "1.0.13"
+# define NET_VERSION "1.0.14"
 #endif /* HAVE_GETADDRINFO && HAVE_GETNAMEINFO */
 
 static SANE_Auth_Callback auth_callback;
@@ -105,6 +120,7 @@ static const SANE_Device **devlist;
 static int client_big_endian; /* 1 == big endian; 0 == little endian */
 static int server_big_endian; /* 1 == big endian; 0 == little endian */
 static int depth; /* bits per pixel */
+static int connect_timeout = -1; /* timeout for connection to saned */
 
 #ifndef NET_USES_AF_INDEP
 static int saned_port;
@@ -148,6 +164,17 @@ add_device (const char *name, Net_Device ** ndp)
 
   DBG (1, "add_device: adding backend %s\n", name);
 
+  for (nd = first_device; nd; nd = nd->next)
+    if (strcmp (nd->name, name) == 0)
+      {
+	DBG (1, "add_device: already in list\n");
+
+	if (ndp)
+	  *ndp = nd;
+
+	return SANE_STATUS_GOOD;
+      }
+
   memset (&hints, 0, sizeof(hints));
 
 # ifdef ENABLE_IPV6
@@ -230,6 +257,17 @@ add_device (const char *name, Net_Device ** ndp)
 
   DBG (1, "add_device: adding backend %s\n", name);
 
+  for (nd = first_device; nd; nd = nd->next)
+    if (strcmp (nd->name, name) == 0)
+      {
+	DBG (1, "add_device: already in list\n");
+
+	if (ndp)
+	  *ndp = nd;
+
+	return SANE_STATUS_GOOD;
+      }
+
   he = gethostbyname (name);
   if (!he)
     {
@@ -290,6 +328,7 @@ connect_dev (Net_Device * dev)
   int on = 1;
   int level = -1;
 #endif
+  struct timeval tv;
 
   int i;
 
@@ -317,6 +356,18 @@ connect_dev (Net_Device * dev)
 	  continue;
 	}
 
+      /* Set SO_SNDTIMEO for the connection to saned */
+      if (connect_timeout > 0)
+	{
+	  tv.tv_sec = connect_timeout;
+	  tv.tv_usec = 0;
+
+	  if (setsockopt (dev->ctl, SOL_SOCKET, SO_SNDTIMEO, &tv, sizeof(tv)) < 0)
+	    {
+	      DBG (1, "connect_dev: [%d] failed to set SO_SNDTIMEO (%s)\n", i, strerror (errno));
+	    }
+	}
+
       if (connect (dev->ctl, addrp->ai_addr, addrp->ai_addrlen) < 0)
 	{
 	  DBG (1, "connect_dev: [%d] failed to connect (%s)\n", i, strerror (errno));
@@ -348,6 +399,7 @@ connect_dev (Net_Device * dev)
   int on = 1;
   int level = -1;
 #endif
+  struct timeval tv;
 
   DBG (2, "connect_dev: trying to connect to %s\n", dev->name);
 
@@ -369,6 +421,19 @@ connect_dev (Net_Device * dev)
   sin = (struct sockaddr_in *) &dev->addr;
   sin->sin_port = saned_port;
 
+
+  /* Set SO_SNDTIMEO for the connection to saned */
+  if (connect_timeout > 0)
+    {
+      tv.tv_sec = connect_timeout;
+      tv.tv_usec = 0;
+
+      if (setsockopt (dev->ctl, SOL_SOCKET, SO_SNDTIMEO, &tv, sizeof(tv)) < 0)
+	{
+	  DBG (1, "connect_dev: failed to set SO_SNDTIMEO (%s)\n", strerror (errno));
+	}
+    }
+
   if (connect (dev->ctl, &dev->addr, sizeof (dev->addr)) < 0)
     {
       DBG (1, "connect_dev: failed to connect (%s)\n", strerror (errno));
@@ -378,6 +443,18 @@ connect_dev (Net_Device * dev)
   DBG (3, "connect_dev: connection succeeded\n");
 #endif /* NET_USES_AF_INDEP */
 
+  /* We're connected now, so reset SO_SNDTIMEO to the default value of 0 */
+  if (connect_timeout > 0)
+    {
+      tv.tv_sec = 0;
+      tv.tv_usec = 0;
+
+      if (setsockopt (dev->ctl, SOL_SOCKET, SO_SNDTIMEO, &tv, sizeof(tv)) < 0)
+	{
+	  DBG (1, "connect_dev: failed to reset SO_SNDTIMEO (%s)\n", strerror (errno));
+	}
+    }
+
 #ifdef TCP_NODELAY
 # ifdef SOL_TCP
   level = SOL_TCP;
@@ -616,10 +693,257 @@ do_authorization (Net_Device * dev, SANE_String resource)
     DBG (1, "do_authorization: auth_active is false... strange\n");
 }
 
+
+#ifdef WITH_AVAHI
+static void
+net_avahi_resolve_callback (AvahiServiceResolver *r, AvahiIfIndex interface, AvahiProtocol protocol,
+			    AvahiResolverEvent event, const char *name, const char *type,
+			    const char *domain, const char *host_name, const AvahiAddress *address,
+			    uint16_t port, AvahiStringList *txt, AvahiLookupResultFlags flags,
+			    void *userdata)
+{
+  char a[AVAHI_ADDRESS_STR_MAX];
+  char *t;
+
+  /* unused */
+  interface = interface;
+  protocol = protocol;
+  userdata = userdata;
+
+  if (!r)
+    return;
+
+  switch (event)
+    {
+      case AVAHI_RESOLVER_FAILURE:
+	DBG (1, "net_avahi_resolve_callback: failed to resolve service '%s' of type '%s' in domain '%s': %s\n",
+	     name, type, domain, avahi_strerror (avahi_client_errno (avahi_service_resolver_get_client (r))));
+	break;
+
+      case AVAHI_RESOLVER_FOUND:
+	DBG (3, "net_avahi_resolve_callback: service '%s' of type '%s' in domain '%s':\n", name, type, domain);
+
+	avahi_address_snprint(a, sizeof (a), address);
+	t = avahi_string_list_to_string (txt);
+
+	DBG (3, "\t%s:%u (%s)\n\tTXT=%s\n\tcookie is %u\n\tis_local: %i\n\tour_own: %i\n"
+	     "\twide_area: %i\n\tmulticast: %i\n\tcached: %i\n",
+	     host_name, port, a, t, avahi_string_list_get_service_cookie (txt),
+	     !!(flags & AVAHI_LOOKUP_RESULT_LOCAL), !!(flags & AVAHI_LOOKUP_RESULT_OUR_OWN),
+	     !!(flags & AVAHI_LOOKUP_RESULT_WIDE_AREA), !!(flags & AVAHI_LOOKUP_RESULT_MULTICAST),
+	     !!(flags & AVAHI_LOOKUP_RESULT_CACHED));
+
+	/* TODO: evaluate TXT record */
+
+	/* Try first with the name */
+	if (add_device (host_name, NULL) != SANE_STATUS_GOOD)
+	  {
+	    DBG (1, "net_avahi_resolve_callback: couldn't add backend with name %s\n", host_name);
+
+	    /* Then try the raw IP address */
+	    if (add_device (t, NULL) != SANE_STATUS_GOOD)
+	      DBG (1, "net_avahi_resolve_callback: couldn't add backend with IP address %s either\n", t);
+	  }
+
+	avahi_free (t);
+	break;
+    }
+
+  avahi_service_resolver_free(r);
+}
+
+static void
+net_avahi_browse_callback (AvahiServiceBrowser *b, AvahiIfIndex interface, AvahiProtocol protocol,
+			   AvahiBrowserEvent event, const char *name, const char *type,
+			   const char *domain, AvahiLookupResultFlags flags, void *userdata)
+{
+  AvahiProtocol proto;
+
+  /* unused */
+  flags = flags;
+  userdata = userdata;
+
+  if (!b)
+    return;
+
+  switch (event)
+    {
+      case AVAHI_BROWSER_FAILURE:
+	DBG (1, "net_avahi_browse_callback: %s\n", avahi_strerror (avahi_client_errno (avahi_service_browser_get_client (b))));
+	avahi_threaded_poll_quit (avahi_thread);
+	return;
+
+      case AVAHI_BROWSER_NEW:
+	DBG (3, "net_avahi_browse_callback: NEW: service '%s' of type '%s' in domain '%s'\n", name, type, domain);
+
+	/* The server will actually be added to our list in the resolver callback */
+
+	/* The resolver object will be freed in the resolver callback, or by
+	 * the server if it terminates before the callback is called.
+	 */
+#ifdef ENABLE_IPV6
+	proto = AVAHI_PROTO_UNSPEC;
+#else
+	proto = AVAHI_PROTO_INET;
+#endif /* ENABLE_IPV6 */
+	if (!(avahi_service_resolver_new (avahi_client, interface, protocol, name, type, domain, proto, 0, net_avahi_resolve_callback, NULL)))
+	  DBG (2, "net_avahi_browse_callback: failed to resolve service '%s': %s\n", name, avahi_strerror (avahi_client_errno (avahi_client)));
+	break;
+
+      case AVAHI_BROWSER_REMOVE:
+	DBG (3, "net_avahi_browse_callback: REMOVE: service '%s' of type '%s' in domain '%s'\n", name, type, domain);
+	/* With the current architecture, we cannot safely remove a server from the list */
+	break;
+
+      case AVAHI_BROWSER_ALL_FOR_NOW:
+      case AVAHI_BROWSER_CACHE_EXHAUSTED:
+	DBG (3, "net_avahi_browse_callback: %s\n", event == AVAHI_BROWSER_CACHE_EXHAUSTED ? "CACHE_EXHAUSTED" : "ALL_FOR_NOW");
+	break;
+    }
+}
+
+static void
+net_avahi_callback (AvahiClient *c, AvahiClientState state, void * userdata)
+{
+  AvahiProtocol proto;
+  int error;
+
+  /* unused */
+  userdata = userdata;
+
+  if (!c)
+    return;
+
+  switch (state)
+    {
+      case AVAHI_CLIENT_CONNECTING:
+	break;
+
+      case AVAHI_CLIENT_S_COLLISION:
+      case AVAHI_CLIENT_S_REGISTERING:
+      case AVAHI_CLIENT_S_RUNNING:
+	if (avahi_browser)
+	  return;
+
+#ifdef ENABLE_IPV6
+	proto = AVAHI_PROTO_UNSPEC;
+#else
+	proto = AVAHI_PROTO_INET;
+#endif /* ENABLE_IPV6 */
+
+	avahi_browser = avahi_service_browser_new (c, AVAHI_IF_UNSPEC, proto, SANED_SERVICE_DNS, NULL, 0, net_avahi_browse_callback, NULL);
+	if (avahi_browser == NULL)
+	  {
+	    DBG (1, "net_avahi_callback: could not create service browser: %s\n", avahi_strerror (avahi_client_errno (c)));
+	    avahi_threaded_poll_quit (avahi_thread);
+	  }
+	break;
+
+      case AVAHI_CLIENT_FAILURE:
+	error = avahi_client_errno (c);
+
+	if (error == AVAHI_ERR_DISCONNECTED)
+	  {
+	    /* Server disappeared - try to reconnect */
+	    avahi_client_free (avahi_client);
+	    avahi_client = NULL;
+
+	    if (avahi_browser)
+	      {
+		avahi_service_browser_free (avahi_browser);
+		avahi_browser = NULL;
+	      }
+
+	    avahi_client = avahi_client_new (avahi_threaded_poll_get (avahi_thread), AVAHI_CLIENT_NO_FAIL, net_avahi_callback, NULL, &error);
+	    if (avahi_client == NULL)
+	      {
+		DBG (1, "net_avahi_init: could not create Avahi client: %s\n", avahi_strerror (error));
+		avahi_threaded_poll_quit (avahi_thread);
+	      }
+	  }
+	else
+	  {
+	    /* Another error happened - game over */
+	    DBG (1, "net_avahi_callback: server connection failure: %s\n", avahi_strerror (error));
+	    avahi_threaded_poll_quit (avahi_thread);
+	  }
+	break;
+    }
+}
+
+
+static void
+net_avahi_init (void)
+{
+  int error;
+
+  avahi_thread = avahi_threaded_poll_new ();
+  if (avahi_thread == NULL)
+    {
+      DBG (1, "net_avahi_init: could not create threaded poll object\n");
+      goto fail;
+    }
+
+  avahi_client = avahi_client_new (avahi_threaded_poll_get (avahi_thread), AVAHI_CLIENT_NO_FAIL, net_avahi_callback, NULL, &error);
+  if (avahi_client == NULL)
+    {
+      DBG (1, "net_avahi_init: could not create Avahi client: %s\n", avahi_strerror (error));
+      goto fail;
+    }
+
+  if (avahi_threaded_poll_start (avahi_thread) < 0)
+    {
+      DBG (1, "net_avahi_init: Avahi thread failed to start\n");
+      goto fail;
+    }
+
+  /* All done */
+  return;
+
+ fail:
+  DBG (1, "net_avahi_init: Avahi init failed, support disabled\n");
+
+  if (avahi_client)
+    {
+      avahi_client_free (avahi_client);
+      avahi_client = NULL;
+    }
+
+  if (avahi_thread)
+    {
+      avahi_threaded_poll_free (avahi_thread);
+      avahi_thread = NULL;
+    }
+}
+
+static void
+net_avahi_cleanup (void)
+{
+  if (!avahi_thread)
+    return;
+
+  DBG (1, "net_avahi_cleanup: stopping thread\n");
+
+  avahi_threaded_poll_stop (avahi_thread);
+
+  if (avahi_browser)
+    avahi_service_browser_free (avahi_browser);
+
+  if (avahi_client)
+    avahi_client_free (avahi_client);
+
+  avahi_threaded_poll_free (avahi_thread);
+
+  DBG (1, "net_avahi_cleanup: done\n");
+}
+#endif /* WITH_AVAHI */
+
+
 SANE_Status
 sane_init (SANE_Int * version_code, SANE_Auth_Callback authorize)
 {
   char device_name[PATH_MAX];
+  const char *optval;
   const char *env;
   size_t len;
   FILE *fp;
@@ -635,9 +959,13 @@ sane_init (SANE_Int * version_code, SANE_Auth_Callback authorize)
   DBG (2, "sane_init: authorize = %p, version_code = %p\n", (void *) authorize,
        (void *) version_code);
 
-  devlist = 0;
-  first_device = 0;
-  first_handle = 0;
+  devlist = NULL;
+  first_device = NULL;
+  first_handle = NULL;
+
+#ifdef WITH_AVAHI
+  net_avahi_init ();
+#endif /* WITH_AVAHI */
 
   auth_callback = authorize;
 
@@ -693,6 +1021,29 @@ sane_init (SANE_Int * version_code, SANE_Auth_Callback authorize)
 	  if (!len)
 	    continue;		/* ignore empty lines */
 
+	  /*
+	   * Check for net backend options.
+	   * Anything that isn't an option is a saned host.
+	   */
+	  if (strstr(device_name, "connect_timeout") != NULL)
+	    {
+	      /* Look for the = sign; if it's not there, error out */
+	      optval = strchr(device_name, '=');
+
+	      if (!optval)
+		continue;
+
+	      optval = sanei_config_skip_whitespace (++optval);
+	      if ((optval != NULL) && (*optval != '\0'))
+		{
+		  connect_timeout = atoi(optval);
+
+		  DBG (2, "sane_init: connect timeout set to %d seconds\n", connect_timeout);
+		}
+
+	      continue;
+	    }
+
 	  DBG (2, "sane_init: trying to add %s\n", device_name);
 	  add_device (device_name, 0);
 	}
@@ -747,6 +1098,15 @@ sane_init (SANE_Int * version_code, SANE_Auth_Callback authorize)
 	DBG (1, "sane_init: not enough memory to duplicate "
 	     "environment variable\n");
     }
+
+  DBG (2, "sane_init: evaluating environment variable SANE_NET_TIMEOUT\n");
+  env = getenv ("SANE_NET_TIMEOUT");
+  if (env)
+    {
+      connect_timeout = atoi(env);
+      DBG (2, "sane_init: connect timeout set to %d seconds from env\n", connect_timeout);
+    }
+
   DBG (2, "sane_init: done\n");
   return SANE_STATUS_GOOD;
 }
@@ -760,6 +1120,10 @@ sane_exit (void)
 
   DBG (1, "sane_exit: exiting\n");
 
+#ifdef WITH_AVAHI
+  net_avahi_cleanup ();
+#endif /* WITH_AVAHI */
+
   /* first, close all handles: */
   for (handle = first_handle; handle; handle = next_handle)
     {
@@ -1000,6 +1364,7 @@ sane_open (SANE_String_Const full_name, SANE_Handle * meta_handle)
   SANE_String nd_name;
   SANE_Status status;
   SANE_Word handle;
+  SANE_Word ack;
   Net_Device *dev;
   Net_Scanner *s;
   int need_auth;
@@ -1224,8 +1589,26 @@ sane_open (SANE_String_Const full_name, SANE_Handle * meta_handle)
   s->next = first_handle;
   s->local_opt.desc = 0;
   s->local_opt.num_options = 0;
+
+  DBG (3, "sane_open: getting option descriptors\n");
+  status = fetch_options (s);
+  if (status != SANE_STATUS_GOOD)
+    {
+      DBG (1, "sane_open: fetch_options failed (%s), closing device again\n",
+	   sane_strstatus (status));
+
+      sanei_w_call (&s->hw->wire, SANE_NET_CLOSE,
+		    (WireCodecFunc) sanei_w_word, &s->handle,
+		    (WireCodecFunc) sanei_w_word, &ack);
+
+      free (s);
+
+      return status;
+    }
+
   first_handle = s;
   *meta_handle = s;
+
   DBG (3, "sane_open: success\n");
   return SANE_STATUS_GOOD;
 }
@@ -1297,7 +1680,7 @@ sane_get_option_descriptor (SANE_Handle handle, SANE_Int option)
 
   if (!s->options_valid)
     {
-      DBG (3, "sane_get_option_descripter: getting option descriptors\n");
+      DBG (3, "sane_get_option_descriptor: getting option descriptors\n");
       status = fetch_options (s);
       if (status != SANE_STATUS_GOOD)
 	{
@@ -1325,21 +1708,16 @@ sane_control_option (SANE_Handle handle, SANE_Int option,
   SANE_Status status;
   size_t value_size;
   int need_auth;
+  SANE_Word local_info;
 
   DBG (3, "sane_control_option: option %d, action %d\n", option, action);
 
   if (!s->options_valid)
     {
-      DBG (3, "sane_control_option: getting option descriptors\n");
-      status = fetch_options (s);
-      if (status != SANE_STATUS_GOOD)
-	{
-	  DBG (1, "sane_control_option: fetch_options failed (%s)\n",
-	       sane_strstatus (status));
-
-	  return status;
-	}
+      DBG (1, "sane_control_option: FRONTEND BUG: option descriptors reload needed\n");
+      return SANE_STATUS_INVAL;
     }
+
   if (((SANE_Word) option >= s->opt.num_options) || (option < 0))
     {
       DBG (1, "sane_control_option: invalid option number\n");
@@ -1366,6 +1744,10 @@ sane_control_option (SANE_Handle handle, SANE_Int option,
       break;
     }
 
+  /* Avoid leaking memory bits */
+  if (value && (action != SANE_ACTION_SET_VALUE))
+    memset (value, 0, value_size);
+
   /* for SET_AUTO the parameter ``value'' is ignored */
   if (action == SANE_ACTION_SET_AUTO)
     value_size = 0;
@@ -1377,6 +1759,8 @@ sane_control_option (SANE_Handle handle, SANE_Int option,
   req.value_size = value_size;
   req.value = value;
 
+  local_info = 0;
+
   DBG (3, "sane_control_option: remote control option\n");
   sanei_w_call (&s->hw->wire, SANE_NET_CONTROL_OPTION,
 		(WireCodecFunc) sanei_w_control_option_req, &req,
@@ -1401,6 +1785,8 @@ sane_control_option (SANE_Handle handle, SANE_Int option,
 	}
       else if (status == SANE_STATUS_GOOD)
 	{
+	  local_info = reply.info;
+
 	  if (info)
 	    *info = reply.info;
 	  if (value_size > 0)
@@ -1422,7 +1808,19 @@ sane_control_option (SANE_Handle handle, SANE_Int option,
     }
   while (need_auth);
 
-  DBG (2, "sane_control_option: done (%s)\n", sane_strstatus (status));
+  DBG (2, "sane_control_option: remote done (%s, info %x)\n", sane_strstatus (status), local_info);
+
+  if ((status == SANE_STATUS_GOOD) && (info == NULL) && (local_info & SANE_INFO_RELOAD_OPTIONS))
+    {
+      DBG (2, "sane_control_option: reloading options as frontend does not care\n");
+
+      status = fetch_options (s);
+
+      DBG (2, "sane_control_option: reload done (%s)\n", sane_strstatus (status));
+    }
+
+  DBG (2, "sane_control_option: done (%s, info %x)\n", sane_strstatus (status), local_info);
+
   return status;
 }
 
@@ -1471,7 +1869,7 @@ sane_start (SANE_Handle handle)
   SANE_Status status;
   int fd, need_auth;
   socklen_t len;
-  u_int16_t port;			/* Internet-specific */
+  uint16_t port;			/* Internet-specific */
 
 
   DBG (3, "sane_start\n");
@@ -1606,7 +2004,7 @@ sane_start (SANE_Handle handle)
   SANE_Status status;
   int fd, need_auth;
   socklen_t len;
-  u_int16_t port;			/* Internet-specific */
+  uint16_t port;			/* Internet-specific */
 
 
   DBG (3, "sane_start\n");
@@ -1756,7 +2154,7 @@ sane_read (SANE_Handle handle, SANE_Byte * data, SANE_Int max_length,
     {
       /* boy, is this painful or what? */
       
-      DBG (4, "sane_read: reading paket length\n");
+      DBG (4, "sane_read: reading packet length\n");
       nread = read (s->data, s->reclen_buf + s->reclen_buf_offset,
 		    4 - s->reclen_buf_offset);
       if (nread < 0)
