@@ -15,7 +15,7 @@
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
-  $Id: niash.c,v 1.13 2006/02/04 10:28:50 ullsig-guest Exp $
+  $Id: niash.c,v 1.14 2008-11-26 21:21:28 kitno-guest Exp $
 */
 
 /*
@@ -834,7 +834,7 @@ sane_init (SANE_Int * piVersion, SANE_Auth_Callback pfnAuth)
 
   if (piVersion != NULL)
     {
-      *piVersion = SANE_VERSION_CODE (V_MAJOR, V_MINOR, BUILD);
+      *piVersion = SANE_VERSION_CODE (SANE_CURRENT_MAJOR, V_MINOR, BUILD);
     }
 
   /* initialise transfer methods */
