@@ -1,6 +1,7 @@
-/* SANE - Scanner Access Now Easy.
+/** SANE - Scanner Access Now Easy.
 
-   Copyright (C) 2006 Wittawat Yamwong <wittawat@web.de>
+   Copyright (C) 2007-2008 Nicolas Martin, <nicols-guest at alioth dot debian dot org>
+   Copyright (C) 2006-2007 Wittawat Yamwong <wittawat@web.de>
 
    This file is part of the SANE package.
 
@@ -40,7 +41,7 @@
    whether to permit this exception to apply to your modifications.
    If you do not wish that, delete this exception notice.
  */
-# include "../include/sane/config.h"
+#include "../include/sane/config.h"
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -48,7 +49,6 @@
 #include <stdarg.h>
 #include <math.h>		/* pow(C90) */
 
-#include <errno.h>		/* POSIX */
 #include <sys/time.h>		/* gettimeofday(4.3BSD) */
 #include <unistd.h>		/* usleep */
 
@@ -66,11 +66,13 @@
 extern const pixma_config_t pixma_mp150_devices[];
 extern const pixma_config_t pixma_mp750_devices[];
 extern const pixma_config_t pixma_mp730_devices[];
+extern const pixma_config_t pixma_iclass_devices[];
 
 static const pixma_config_t *const pixma_devices[] = {
   pixma_mp150_devices,
   pixma_mp750_devices,
   pixma_mp730_devices,
+  pixma_iclass_devices,
   NULL
 };
 
@@ -79,6 +81,7 @@ static time_t tstart_sec = 0;
 static uint32_t tstart_usec = 0;
 static int debug_level = 1;
 
+
 #ifndef NDEBUG
 
 static void
@@ -86,7 +89,8 @@ u8tohex (uint8_t x, char *str)
 {
   static const char hdigit[16] =
     { '0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'a', 'b', 'c', 'd',
-'e', 'f' };
+    'e', 'f'
+    };
   str[0] = hdigit[(x >> 4) & 0xf];
   str[1] = hdigit[x & 0xf];
   str[2] = '\0';
@@ -105,34 +109,41 @@ void
 pixma_hexdump (int level, const void *d_, unsigned len)
 {
   const uint8_t *d = (const uint8_t *) (d_);
-  unsigned ofs, c;
+  unsigned ofs, c, plen;
   char line[100];		/* actually only 1+8+1+8*3+1+8*3+1 = 61 bytes needed */
 
   if (level > debug_level)
     return;
+  if (level == debug_level)
+    /* if debuglevel == exact match and buffer contains more than 3 lines, print 2 lines + .... */
+    plen = (len > 64) ? 32: len;
+  else
+    plen = len;
   ofs = 0;
-  while (ofs < len)
+  while (ofs < plen)
     {
       char *p;
       line[0] = ' ';
       u32tohex (ofs, line + 1);
       line[9] = ':';
       p = line + 10;
-      for (c = 0; c != 16 && (ofs + c) < len; c++)
-	{
-	  u8tohex (d[ofs + c], p);
-	  p[2] = ' ';
-	  p += 3;
-	  if (c == 7)
-	    {
-	      p[0] = ' ';
-	      p++;
-	    }
-	}
+      for (c = 0; c != 16 && (ofs + c) < plen; c++)
+        {
+          u8tohex (d[ofs + c], p);
+          p[2] = ' ';
+          p += 3;
+          if (c == 7)
+            {
+              p[0] = ' ';
+              p++;
+            }
+        }
       p[0] = '\0';
       pixma_dbg (level, "%s\n", line);
       ofs += c;
     }
+  if (len > plen)
+    pixma_dbg(level, "......\n");
 }
 
 static void
@@ -180,13 +191,55 @@ pixma_dump (int level, const char *type, const void *data, int len,
 	pixma_dbg (level, " ...\n");
     }
   if (len < 0)
-    pixma_dbg (level, "  ERROR: %s\n", strerror (-len));
+    pixma_dbg (level, "  ERROR: %s\n", pixma_strerror (len));
   pixma_dbg (level, "\n");
 }
 
 
 #endif /* NDEBUG */
 
+/* NOTE: non-reentrant */
+const char *
+pixma_strerror (int error)
+{
+  static char buf[50];
+
+  /* TODO: more human friendly messages */
+  switch (error)
+    {
+    case PIXMA_EIO:
+      return "EIO";
+    case PIXMA_ENODEV:
+      return "ENODEV";
+    case PIXMA_EACCES:
+      return "EACCES";
+    case PIXMA_ENOMEM:
+      return "ENOMEM";
+    case PIXMA_EINVAL:
+      return "EINVAL";
+    case PIXMA_EBUSY:
+      return "EBUSY";
+    case PIXMA_ECANCELED:
+      return "ECANCELED";
+    case PIXMA_ENOTSUP:
+      return "ENOTSUP";
+    case PIXMA_ETIMEDOUT:
+      return "ETIMEDOUT";
+    case PIXMA_EPROTO:
+      return "EPROTO";
+    case PIXMA_EPAPER_JAMMED:
+      return "EPAPER_JAMMED";
+    case PIXMA_ECOVER_OPEN:
+      return "ECOVER_OPEN";
+    case PIXMA_ENO_PAPER:
+      return "ENO_PAPER";
+    case PIXMA_EOF:
+      return "EEOF";
+    }
+  snprintf (buf, sizeof (buf), "EUNKNOWN:%d", error);
+  return buf;
+}
+
 void
 pixma_set_debug_level (int level)
 {
@@ -257,11 +310,11 @@ pixma_map_status_errno (unsigned status)
     case PIXMA_STATUS_OK:
       return 0;
     case PIXMA_STATUS_FAILED:
-      return -ECANCELED;
+      return PIXMA_ECANCELED;
     case PIXMA_STATUS_BUSY:
-      return -EBUSY;
+      return PIXMA_EBUSY;
     default:
-      return -EPROTO;
+      return PIXMA_EPROTO;
     }
 }
 
@@ -282,30 +335,30 @@ pixma_check_result (pixma_cmdbuf_t * cb)
     {
       error = pixma_map_status_errno (pixma_get_be16 (r));
       if (expected_reslen != 0)
-	{
-	  if (len == expected_reslen)
-	    {
-	      if (pixma_sum_bytes (r + header_len, len - header_len) != 0)
-		error = -EPROTO;
-	    }
-	  else
-	    {
-	      /* This case will happen when a command cannot be completely
-	         executed, e.g. because you press the cancel button. The
-	         device will return only a header with PIXMA_STATUS_FAILED. */
-	      if (len != header_len)
-		error = -EPROTO;
-	    }
-	}
+        {
+          if (len == expected_reslen)
+            {
+              if (pixma_sum_bytes (r + header_len, len - header_len) != 0)
+                 error = PIXMA_EPROTO;
+            }
+          else
+            {
+              /* This case will happen when a command cannot be completely
+                 executed, e.g. because you press the cancel button. The
+                 device will return only a header with PIXMA_STATUS_FAILED. */
+              if (len != header_len)
+                 error = PIXMA_EPROTO;
+            }
+        }
     }
   else
-    error = -EPROTO;
+    error = PIXMA_EPROTO;
 
 #ifndef NDEBUG
-  if (error == -EPROTO)
+  if (error == PIXMA_EPROTO)
     {
-      pixma_dbg (1, "WARNING: result len=%d expected %d: %s\n",
-		 len, cb->expected_reslen, strerror (-error));
+      pixma_dbg (1, "WARNING: result len=%d expected %d\n",
+		 len, cb->expected_reslen);
       pixma_hexdump (1, r, MIN (len, 64));
     }
 #endif
@@ -320,7 +373,17 @@ pixma_cmd_transaction (pixma_t * s, const void *cmd, unsigned cmdlen,
 
   error = pixma_write (s->io, cmd, cmdlen);
   if (error != (int) cmdlen)
-    return error;		/* FIXME: make sure that error < 0! */
+    {
+      if (error >= 0)
+        {
+          /* Write timeout is too low? */
+          PDBG (pixma_dbg
+          (1, "ERROR:incomplete write, %u out of %u written\n",
+           (unsigned) error, cmdlen));
+          error = PIXMA_ETIMEDOUT;
+        }
+      return error;
+    }
 
   /* When you send the start_session command while the scanner optic is
      going back to the home position after the last scan session has been
@@ -334,19 +397,17 @@ pixma_cmd_transaction (pixma_t * s, const void *cmd, unsigned cmdlen,
   do
     {
       error = pixma_read (s->io, data, expected_len);
-      if (error == -ETIMEDOUT)
-	PDBG (pixma_dbg (2, "No response yet. Timed out in %d sec.\n", tmo));
+      if (error == PIXMA_ETIMEDOUT)
+        PDBG (pixma_dbg (2, "No response yet. Timed out in %d sec.\n", tmo));
     }
-  while (error == -ETIMEDOUT && --tmo != 0);
+  while (error == PIXMA_ETIMEDOUT && --tmo != 0);
   if (error < 0)
     {
       PDBG (pixma_dbg (1, "WARNING:Error in response phase. cmd:%02x%02x\n",
 		       ((const uint8_t *) cmd)[0],
 		       ((const uint8_t *) cmd)[1]));
-      PDBG (pixma_dbg
-	    (1,
-	     "   If the scanner hangs, reset it and/or unplug the "
-	     "USB cable.\n"));
+      PDBG (pixma_dbg (1,"  If the scanner hangs, reset it and/or unplug the "
+	                       "USB cable.\n"));
     }
   return error;			/* length of the result packet or error */
 }
@@ -396,7 +457,7 @@ pixma_check_dpi (unsigned dpi, unsigned max)
   /* valid dpi = 75 * 2^n */
   unsigned temp = dpi / 75;
   if (dpi > max || dpi < 75 || 75 * temp != dpi || (temp & (temp - 1)) != 0)
-    return -EINVAL;
+    return PIXMA_EINVAL;
   return 0;
 }
 
@@ -430,12 +491,12 @@ pixma_open (unsigned devnr, pixma_t ** handle)
   *handle = NULL;
   cfg = pixma_get_device_config (devnr);
   if (!cfg)
-    return -EINVAL;		/* invalid devnr */
+    return PIXMA_EINVAL;	/* invalid devnr */
   PDBG (pixma_dbg (2, "pixma_open(): %s\n", cfg->name));
 
   s = (pixma_t *) calloc (1, sizeof (s[0]));
   if (!s)
-    return -ENOMEM;
+    return PIXMA_ENOMEM;
   s->next = first_pixma;
   first_pixma = s;
 
@@ -443,19 +504,24 @@ pixma_open (unsigned devnr, pixma_t ** handle)
   error = pixma_connect (devnr, &s->io);
   if (error < 0)
     {
-      PDBG (pixma_dbg (2, "pixma_connect() failed:%s\n", strerror (-error)));
+      PDBG (pixma_dbg
+	    (2, "pixma_connect() failed %s\n", pixma_strerror (error)));
       goto rollback;
     }
-  strncpy (s->id, pixma_get_device_id (devnr), sizeof (s->id));
+  strncpy (s->id, pixma_get_device_id (devnr), sizeof (s->id) - 1);
   s->ops = s->cfg->ops;
   s->scanning = 0;
   error = s->ops->open (s);
+  if (error < 0)
+    goto rollback;
+  error = pixma_deactivate (s->io);
   if (error < 0)
     goto rollback;
   *handle = s;
   return 0;
 
 rollback:
+  PDBG (pixma_dbg (2, "pixma_open() failed %s\n", pixma_strerror (error)));
   pixma_close (s);
   return error;
 }
@@ -523,7 +589,8 @@ pixma_scan (pixma_t * s, pixma_scan_param_t * sp)
     }
   else
     {
-      PDBG (pixma_dbg (3, "pixma_scan() failed:%s\n", strerror (-error)));
+      PDBG (pixma_dbg
+	    (3, "pixma_scan() failed %s\n", pixma_strerror (error)));
     }
 
   return error;
@@ -554,7 +621,7 @@ pixma_read_image (pixma_t * s, void *buf, unsigned len)
     return 0;
   if (s->cancel)
     {
-      result = -ECANCELED;
+      result = PIXMA_ECANCELED;
       goto cancel;
     }
 
@@ -565,67 +632,65 @@ pixma_read_image (pixma_t * s, void *buf, unsigned len)
   if (s->underrun)
     {
       if (s->cur_image_size < s->param->image_size)
-	{
-	  ib.wptr = fill_pixels (s, ib.wptr, ib.wend, 0xff);
-	}
+        {
+          ib.wptr = fill_pixels (s, ib.wptr, ib.wend, 0xff);
+        }
       else
-	{
-	  PDBG (pixma_dbg
-		(3, "pixma_read_image():completed (underrun detected)\n"));
-	  s->scanning = 0;
-	}
+        {
+          PDBG (pixma_dbg
+          (3, "pixma_read_image():completed (underrun detected)\n"));
+          s->scanning = 0;
+        }
       return ib.wptr - (uint8_t *) buf;
     }
 
   while (ib.wptr != ib.wend)
     {
       if (ib.rptr == ib.rend)
-	{
-	  ib.rptr = ib.rend = NULL;
-	  result = s->ops->fill_buffer (s, &ib);
-	  if (result < 0)
-	    goto cancel;
-	  if (result == 0)
-	    {			/* end of image? */
-	      s->ops->finish_scan (s);
-#ifndef NDEBUG
-	      if (s->cur_image_size != s->param->image_size)
-		{
-		  pixma_dbg (1, "WARNING:image size mismatches\n");
-		  pixma_dbg (1,
-			     "    %u expected (%d lines) but %u received (%d lines)\n",
-			     s->param->image_size, s->param->h,
-			     s->cur_image_size,
-			     s->cur_image_size / s->param->line_size);
-		  if ((s->cur_image_size % s->param->line_size) != 0)
-		    {
-		      pixma_dbg (1,
-				 "BUG:received data not multiple of line_size\n");
-		    }
-		}
-#endif /* !NDEBUG */
-	      if (s->cur_image_size < s->param->image_size)
-		{
-		  s->underrun = 1;
-		  ib.wptr = fill_pixels (s, ib.wptr, ib.wend, 0xff);
-		}
-	      else
-		{
-		  PDBG (pixma_dbg (3, "pixma_read_image():completed\n"));
-		  s->scanning = 0;
-		}
-	      break;
-	    }
-	  s->cur_image_size += result;
-	  PASSERT (s->cur_image_size <= s->param->image_size);
-	}
+        {
+          ib.rptr = ib.rend = NULL;
+          result = s->ops->fill_buffer (s, &ib);
+          if (result < 0)
+            goto cancel;
+          if (result == 0)
+            {			/* end of image? */
+              s->ops->finish_scan (s);
+              if (s->cur_image_size != s->param->image_size)
+                {
+                  pixma_dbg (1, "WARNING:image size mismatches\n");
+                  pixma_dbg (1,
+                       "    %u expected (%d lines) but %u received (%d lines)\n",
+                       s->param->image_size, s->param->h,
+                       s->cur_image_size,
+                       s->cur_image_size / s->param->line_size);
+                  if ((s->cur_image_size % s->param->line_size) != 0)
+                    {
+                      pixma_dbg (1,
+                     "BUG:received data not multiple of line_size\n");
+                    }
+                }
+              if (s->cur_image_size < s->param->image_size)
+                {
+                  s->underrun = 1;
+                  ib.wptr = fill_pixels (s, ib.wptr, ib.wend, 0xff);
+                }
+              else
+                {
+                  PDBG (pixma_dbg (3, "pixma_read_image():completed\n"));
+                  s->scanning = 0;
+                }
+              break;
+            }
+          s->cur_image_size += result;
+          PASSERT (s->cur_image_size <= s->param->image_size);
+        }
       if (ib.rptr)
-	{
-	  unsigned count = MIN (ib.rend - ib.rptr, ib.wend - ib.wptr);
-	  memcpy (ib.wptr, ib.rptr, count);
-	  ib.rptr += count;
-	  ib.wptr += count;
-	}
+        {
+          unsigned count = MIN (ib.rend - ib.rptr, ib.wend - ib.wptr);
+          memcpy (ib.wptr, ib.rptr, count);
+          ib.rptr += count;
+          ib.wptr += count;
+        }
     }
   s->imagebuf = ib;		/* store rptr and rend */
   return ib.wptr - (uint8_t *) buf;
@@ -633,11 +698,16 @@ pixma_read_image (pixma_t * s, void *buf, unsigned len)
 cancel:
   s->ops->finish_scan (s);
   s->scanning = 0;
-  if (result == -ECANCELED)
+  if (result == PIXMA_ECANCELED)
     {
       PDBG (pixma_dbg (3, "pixma_read_image():cancelled by %sware\n",
 		       (s->cancel) ? "soft" : "hard"));
     }
+  else
+    {
+      PDBG (pixma_dbg (3, "pixma_read_image() failed %s\n",
+		       pixma_strerror (result)));
+    }
   return result;
 }
 
@@ -653,7 +723,19 @@ pixma_enable_background (pixma_t * s, int enabled)
   return pixma_set_interrupt_mode (s->io, enabled);
 }
 
-unsigned
+int
+pixma_activate_connection(pixma_t * s)
+{
+  return pixma_activate (s->io);
+}
+
+int
+pixma_deactivate_connection(pixma_t * s)
+{
+  return pixma_deactivate (s->io);
+}
+
+uint32_t
 pixma_wait_event (pixma_t * s, int timeout /*ms */ )
 {
   unsigned events;
@@ -677,34 +759,75 @@ pixma_check_scan_param (pixma_t * s, pixma_scan_param_t * sp)
 {
   if (!(sp->channels == 3 ||
 	(sp->channels == 1 && (s->cfg->cap & PIXMA_CAP_GRAY) != 0)))
-    return -EINVAL;
+    return PIXMA_EINVAL;
 
   if (pixma_check_dpi (sp->xdpi, s->cfg->xdpi) < 0 ||
       pixma_check_dpi (sp->ydpi, s->cfg->ydpi) < 0)
-    return -EINVAL;
+    return PIXMA_EINVAL;
 
   /* xdpi must be equal to ydpi except that
      xdpi = max_xdpi and ydpi = max_ydpi. */
   if (!(sp->xdpi == sp->ydpi ||
 	(sp->xdpi == s->cfg->xdpi && sp->ydpi == s->cfg->ydpi)))
-    return -EINVAL;
+    return PIXMA_EINVAL;
+
+  if (s->ops->check_param (s, sp) < 0)
+    return PIXMA_EINVAL;
 
   /* FIXME: I assume the same minimum width and height for every model. */
   CLAMP2 (sp->x, sp->w, 13, s->cfg->width, sp->xdpi);
   CLAMP2 (sp->y, sp->h, 8, s->cfg->height, sp->ydpi);
 
-  if (!(s->cfg->cap & PIXMA_CAP_ADF))
-    sp->source = PIXMA_SOURCE_FLATBED;
+  switch (sp->source)
+    {
+    case PIXMA_SOURCE_FLATBED:
+      break;
+
+    case PIXMA_SOURCE_TPU:
+      if ((s->cfg->cap & PIXMA_CAP_TPU) != PIXMA_CAP_TPU)
+        {
+          sp->source = PIXMA_SOURCE_FLATBED;
+          PDBG (pixma_dbg
+          (1, "WARNING: TPU unsupported, fallback to flatbed.\n"));
+        }
+      break;
+
+    case PIXMA_SOURCE_ADF:
+      if ((s->cfg->cap & PIXMA_CAP_ADF) != PIXMA_CAP_ADF)
+        {
+          sp->source = PIXMA_SOURCE_FLATBED;
+          PDBG (pixma_dbg
+          (1, "WARNING: ADF unsupported, fallback to flatbed.\n"));
+        }
+      break;
+
+    case PIXMA_SOURCE_ADFDUP:
+      if ((s->cfg->cap & PIXMA_CAP_ADFDUP) != PIXMA_CAP_ADFDUP)
+        {
+          if (s->cfg->cap & PIXMA_CAP_ADF)
+            {
+              sp->source = PIXMA_SOURCE_ADF;
+            }
+          else
+            {
+              sp->source = PIXMA_SOURCE_FLATBED;
+            }
+          PDBG (pixma_dbg
+          (1, "WARNING: ADF duplex unsupported, fallback to %d.\n",
+           sp->source));
+        }
+      break;
+    }
 
   if (sp->depth == 0)
     sp->depth = 8;
   if ((sp->depth % 8) != 0)
-    return -EINVAL;
+    return PIXMA_EINVAL;
 
   sp->line_size = 0;
 
   if (s->ops->check_param (s, sp) < 0)
-    return -EINVAL;
+    return PIXMA_EINVAL;
 
   if (sp->line_size == 0)
     sp->line_size = sp->depth / 8 * sp->channels * sp->w;
@@ -748,9 +871,9 @@ pixma_fill_gamma_table (double gamma, uint8_t * table, unsigned n)
 }
 
 int
-pixma_find_scanners (void)
+pixma_find_scanners (const char **conf_devices)
 {
-  return pixma_collect_devices (pixma_devices);
+  return pixma_collect_devices (conf_devices, pixma_devices);
 }
 
 const char *
@@ -765,7 +888,7 @@ int
 pixma_get_device_status (pixma_t * s, pixma_device_status_t * status)
 {
   if (!status)
-    return -EINVAL;
+    return PIXMA_EINVAL;
   memset (status, 0, sizeof (*status));
   return s->ops->get_status (s, status);
 }
