@@ -1,8 +1,3 @@
-/*.............................................................................
- * Project : SANE library for Plustek flatbed scanners.
- *.............................................................................
- */
-
 /** @file plustek-pp.h
  *  @brief Definitions for the backend.
  *
@@ -15,6 +10,9 @@
  *
  * History:
  * - 0.01 - initial version
+ * - 0.43 - bumped up version to reflect the former module code version
+ *        - removed Version from ScannerCaps
+ *        - added _E_FAULT
  * .
  * <hr>
  * This file is part of the SANE package.
@@ -238,6 +236,12 @@ typedef const struct mode_param
 #define _E_SEQUENCE	  (_FIRST_ERR-30)	/* caller sequence does not match	*/
 #define _E_NO_ASIC	  (_FIRST_ERR-31)	/* can't detect ASIC            	*/
 
+#ifdef __KERNEL__
+# define _E_FAULT     (-EFAULT)
+#else
+# define _E_FAULT     (_E_INTERNAL)    /* should never happen in userspace  */
+#endif
+
 #define _E_LAMP_NOT_IN_POS	(_FIRST_ERR-40)
 #define _E_LAMP_NOT_STABLE	(_FIRST_ERR-41)
 #define _E_NODATA           (_FIRST_ERR-42)
@@ -348,9 +352,9 @@ typedef const struct mode_param
 #define _PTDRV_GET_CROPINFO 	_IOR('x', 5, CropInfo)		/* get crop		 */
 #define _PTDRV_SET_ENV 			_IOWR('x',6, ScanInfo)		/* set env.		 */
 #define _PTDRV_START_SCAN 		_IOR('x', 7, StartScan)		/* start scan 	 */
-#define _PTDRV_STOP_SCAN 		_IOWR('x', 8, int)			/* stop scan 	 */
+#define _PTDRV_STOP_SCAN 		_IOWR('x', 8, short)		/* stop scan 	 */
 #define _PTDRV_CLOSE_DEVICE 	_IO('x',  9)				/* close 		 */
-#define _PTDRV_ACTION_BUTTON	_IOR('x', 10, int)	 		/* rd act. button*/
+#define _PTDRV_ACTION_BUTTON	_IOR('x', 10, unsigned char)/* rd act. button*/
 #define _PTDRV_ADJUST           _IOR('x', 11, AdjDef)		/* adjust driver */
 #define _PTDRV_SETMAP           _IOR('x', 12, MapDef)		/* download gamma*/
 
@@ -421,14 +425,13 @@ typedef struct {
 } RANGE, *PRANGE;
 
 typedef struct {
-    RANGE	    	rDataType;      /* available scan modes 			*/
-    unsigned long	dwFlag;		    /* refer to SECTION (1.2)           */
-    unsigned short	wIOBase;		/* refer to SECTION (1.3)			*/
-    unsigned short	wMaxExtentX;	/* scanarea width					*/
-    unsigned short	wMaxExtentY;	/* scanarea height					*/
-    unsigned short	AsicID;  		/* copy of RegAsicID 				*/
-  	unsigned short	Model;  		/* model as best we can determine 	*/
-    unsigned short	Version;		/* drivers version					*/
+	RANGE          rDataType;   /* available scan modes 			*/
+	unsigned long  dwFlag;      /* refer to SECTION (1.2)           */
+	unsigned short wIOBase;     /* refer to SECTION (1.3)			*/
+	unsigned short wMaxExtentX; /* scanarea width					*/
+	unsigned short wMaxExtentY; /* scanarea height					*/
+	unsigned short AsicID;      /* copy of RegAsicID 				*/
+	unsigned short Model;       /* model as best we can determine 	*/
 } ScannerCaps, *pScannerCaps;
 
 typedef struct {
@@ -576,9 +579,9 @@ typedef struct Plustek_Device
     int  (*putImgInfo) ( struct Plustek_Device*, pImgDef    );
     int  (*setScanEnv) ( struct Plustek_Device*, pScanInfo  );
     int  (*setMap)     ( struct Plustek_Device*, SANE_Word*,
-												 SANE_Word, SANE_Word );
+	                                             SANE_Word, SANE_Word );
     int  (*startScan)  ( struct Plustek_Device*, pStartScan );
-    int  (*stopScan)   ( struct Plustek_Device*, int* );
+    int  (*stopScan)   ( struct Plustek_Device*, short* );
     int  (*readImage)  ( struct Plustek_Device*, SANE_Byte*, unsigned long );
 
     int  (*prepare)    ( struct Plustek_Device*, SANE_Byte* );
@@ -599,7 +602,7 @@ typedef union
 typedef struct Plustek_Scanner
 {
     struct Plustek_Scanner *next;
-    pid_t 					reader_pid;		/* process id of reader          */
+    SANE_Pid					reader_pid;		/* process id of reader          */
     SANE_Status             exit_code;      /* status of the reader process  */
 	int                     r_pipe;         /* pipe to reader process        */
 	int                     w_pipe;         /* pipe from reader process      */
