@@ -11,7 +11,7 @@
  *
  * original code taken from sane-0.71<br>
  * Copyright (C) 1997 Hypercore Software Design, Ltd.<br>
- * Copyright (C) 2001-2005 Gerhard Jaeger <gerhard@gjaeger.de>
+ * Copyright (C) 2001-2007 Gerhard Jaeger <gerhard@gjaeger.de>
  *
  * History:
  * - 0.30 - initial version
@@ -58,6 +58,8 @@
  * - 0.50 - cleanup
  *        - added OPT_SPEEDUP
  * - 0.51 - added OPT_CALIBRATE
+ * - 0.52 - added skipDarkStrip and incDarkTgt to struct AdjDef
+ *        - added OPT_LOFF4DARK
  * .
  * <hr>
  * This file is part of the SANE package.
@@ -225,6 +227,7 @@ enum {
 	OPT_LAMPOFF_TIMER,
 	OPT_LAMPOFF_ONEND,
 	OPT_WARMUPTIME,
+	OPT_LOFF4DARK,
 	OPT_CACHECAL,
 	OPT_SPEEDUP,
 	OPT_CALIBRATE,
@@ -263,9 +266,11 @@ typedef struct {
 	int     lampOffOnEnd;
 	int     warmup;
 	int     enableTpa;
-	int     skipCalibration;
+	int     skipCalibration;  /**< skip entire calibration */
 	int     skipFine;
 	int     skipFineWhite;
+	int     skipDarkStrip;
+	int     incDarkTgt;
 	int     disableSpeedup;
 	int     invertNegatives;
 	int     cacheCalData;
@@ -378,7 +383,7 @@ typedef union
 typedef struct Plustek_Scanner
 {
 	struct Plustek_Scanner *next;
-	pid_t                   reader_pid;     /* process id of reader          */
+	SANE_Pid                reader_pid;     /* process id of reader          */
 	SANE_Status             exit_code;      /* status of the reader process  */
 	int                     r_pipe;         /* pipe to reader process        */
 	int                     w_pipe;         /* pipe from reader process      */
