@@ -578,7 +578,7 @@ attach (const char *devname, S9036_Device ** devp)
      strings, but ?  At offset 36 my SIEMENS scanner identifies as an
      AGFA one ?!   */
 
-  if (result[0] != 6 || strncmp (result + 36, "AGFA03", 6))
+  if (result[0] != 6 || strncmp ((char *)result + 36, "AGFA03", 6))
     {
       DBG (1, "attach: device doesn't look like a Siemens 9036 scanner\n");
       return SANE_STATUS_INVAL;
@@ -680,6 +680,7 @@ init_options (S9036_Scanner * s)
 
   s->opt[OPT_NUM_OPTS].title = SANE_TITLE_NUM_OPTIONS;
   s->opt[OPT_NUM_OPTS].desc = SANE_DESC_NUM_OPTIONS;
+  s->opt[OPT_NUM_OPTS].type = SANE_TYPE_INT;
   s->opt[OPT_NUM_OPTS].cap = SANE_CAP_SOFT_DETECT;
   s->val[OPT_NUM_OPTS] = NUM_OPTIONS;
 
@@ -833,7 +834,7 @@ sane_init (SANE_Int * version_code, SANE_Auth_Callback authorize)
   DBG_INIT ();
 
   if (version_code)
-    *version_code = SANE_VERSION_CODE (V_MAJOR, V_MINOR, 0);
+    *version_code = SANE_VERSION_CODE (SANE_CURRENT_MAJOR, V_MINOR, 0);
 
   fp = sanei_config_open ("s9036.conf");
   if (!fp)
