@@ -43,7 +43,7 @@
 */
 
 /* 
-   $Id: sceptre.c,v 1.13 2005/09/19 19:57:48 fzago-guest Exp $
+   $Id: sceptre.c,v 1.14 2008-11-26 21:21:29 kitno-guest Exp $
    Sceptre S1200 SCSI scanner (sometimes also called S120)
 */
 
@@ -1317,13 +1317,13 @@ sane_init (SANE_Int * version_code, SANE_Auth_Callback __sane_unused__ authorize
 
   DBG (DBG_proc, "sane_init: enter\n");
 
-  DBG (DBG_error, "This is sane-sceptre version %d.%d-%d\n", V_MAJOR,
+  DBG (DBG_error, "This is sane-sceptre version %d.%d-%d\n", SANE_CURRENT_MAJOR,
        V_MINOR, BUILD);
   DBG (DBG_error, "(C) 2002 by Frank Zago\n");
 
   if (version_code)
     {
-      *version_code = SANE_VERSION_CODE (V_MAJOR, V_MINOR, BUILD);
+      *version_code = SANE_VERSION_CODE (SANE_CURRENT_MAJOR, V_MINOR, BUILD);
     }
 
   fp = sanei_config_open (SCEPTRE_CONFIG_FILE);
