@@ -1,4 +1,4 @@
-static const char RCSid[] = "$Header: /cvsroot/sane/sane-backends/backend/sp15c.c,v 1.12 2005/10/01 17:06:25 hmg-guest Exp $";
+static const char RCSid[] = "$Header: /cvsroot/sane/sane-backends/backend/sp15c.c,v 1.17 2008-11-26 21:21:30 kitno-guest Exp $";
 /* sane - Scanner Access Now Easy.
 
    This file is part of the SANE package.
@@ -45,6 +45,25 @@ static const char RCSid[] = "$Header: /cvsroot/sane/sane-backends/backend/sp15c.
 
 /*
  * $Log: sp15c.c,v $
+ * Revision 1.17  2008-11-26 21:21:30  kitno-guest
+ * * backend/*.[ch]: nearly every backend used V_MAJOR
+ * instead of SANE_CURRENT_MAJOR in sane_init()
+ * * backend/snapscan.c: remove EXPECTED_VERSION check
+ * since new SANE standard is forward compatible
+ *
+ * Revision 1.16  2008-07-26 03:53:44  kitno-guest
+ * separate x-resolution from resolution, and update all backends that use
+ * it, to prevent ui change
+ *
+ * Revision 1.15  2007-11-18 10:59:18  ellert-guest
+ * Fix handling of valid "negative" PIDs
+ *
+ * Revision 1.14  2007-10-26 14:56:38  jblache
+ * OPT_NUM_OPTS must be of type SANE_TYPE_INT.
+ *
+ * Revision 1.13  2006/03/29 20:48:50  hmg-guest
+ * Fixed ADF support. Patch from Andreas Degert <ad@papyrus-gmbh.de>.
+ *
  * Revision 1.12  2005/10/01 17:06:25  hmg-guest
  * Fixed some warnings (bug #302290).
  *
@@ -266,7 +285,7 @@ sane_init (SANE_Int * version_code, SANE_Auth_Callback authorize)
   sanei_thread_init ();
 
   if (version_code)
-    *version_code = SANE_VERSION_CODE (V_MAJOR, V_MINOR, 0);
+    *version_code = SANE_VERSION_CODE (SANE_CURRENT_MAJOR, V_MINOR, 0);
   fp = sanei_config_open (SP15C_CONFIG_FILE);
   if (!fp)
     {
@@ -1382,6 +1401,7 @@ init_options (struct sp15c *scanner)
 
   scanner->opt[OPT_NUM_OPTS].title = SANE_TITLE_NUM_OPTIONS;
   scanner->opt[OPT_NUM_OPTS].desc = SANE_DESC_NUM_OPTIONS;
+  scanner->opt[OPT_NUM_OPTS].type = SANE_TYPE_INT;
   scanner->opt[OPT_NUM_OPTS].cap = SANE_CAP_SOFT_DETECT;
 
 /************** "Mode" group: **************/
@@ -1431,7 +1451,7 @@ init_options (struct sp15c *scanner)
   scanner->opt[OPT_PRESCAN].unit = SANE_UNIT_NONE;
 
   /* resolution */
-  scanner->opt[OPT_X_RES].name = SANE_NAME_SCAN_X_RESOLUTION;
+  scanner->opt[OPT_X_RES].name = SANE_NAME_SCAN_RESOLUTION;
   scanner->opt[OPT_X_RES].title = SANE_TITLE_SCAN_X_RESOLUTION;
   scanner->opt[OPT_X_RES].desc = SANE_DESC_SCAN_X_RESOLUTION;
   scanner->opt[OPT_X_RES].type = SANE_TYPE_INT;
@@ -1737,11 +1757,10 @@ do_cancel (struct sp15c *scanner)
 {
   DBG (10, "do_cancel\n");
   swap_res (scanner);
-  scanner->scanning = SANE_FALSE;
 
   do_eof (scanner);             /* close pipe and reposition scanner */
 
-  if (scanner->reader_pid > 0)
+  if (scanner->reader_pid != -1)
     {
       int exit_status;
       DBG (10, "do_cancel: kill reader_process\n");
@@ -1749,7 +1768,7 @@ do_cancel (struct sp15c *scanner)
       sanei_thread_kill (scanner->reader_pid);
       DBG (50, "wait for scanner to stop\n");
       sanei_thread_waitpid (scanner->reader_pid, &exit_status);
-      scanner->reader_pid = 0;
+      scanner->reader_pid = -1;
     }
 
   if (scanner->sfd >= 0)
@@ -2061,6 +2080,7 @@ do_eof (struct sp15c *scanner)
 {
   DBG (10, "do_eof\n");
 
+  scanner->scanning = SANE_FALSE;
   if (scanner->pipe >= 0)
     {
       close (scanner->pipe);
