@@ -50,6 +50,8 @@
 #include <unistd.h>
 #include <sys/time.h>
 
+#include "_stdint.h"
+
 #include "sane/sane.h"
 #include "sane/sanei.h"
 #include "sane/saneopts.h"
@@ -117,22 +119,22 @@ static const SANE_Range abs_percentage_range =
 
 
 #define INQ_LEN	0x60
-static const u_int8_t inquiry[] =
+static const uint8_t inquiry[] =
 {
   TAMARACK_SCSI_INQUIRY, 0x00, 0x00, 0x00, INQ_LEN, 0x00
 };
 
-static const u_int8_t test_unit_ready[] =
+static const uint8_t test_unit_ready[] =
 {
   TAMARACK_SCSI_TEST_UNIT_READY, 0x00, 0x00, 0x00, 0x00, 0x00
 };
 
-static const u_int8_t stop[] =
+static const uint8_t stop[] =
 {
   TAMARACK_SCSI_START_STOP, 0x00, 0x00, 0x00, 0x00, 0x00
 };
 
-static const u_int8_t get_status[] =
+static const uint8_t get_status[] =
 {
   TAMARACK_SCSI_GET_DATA_STATUS, 0x00, 0x00, 0x00, 0x00, 0x00, 
                                        0x00, 0x00, 0x0c, 0x00
@@ -465,14 +467,14 @@ do_cancel (Tamarack_Scanner *s)
 
   do_eof (s);
 
-  if (s->reader_pid > 0)
+  if (s->reader_pid != -1)
     {
       int exit_status;
 
       /* ensure child knows it's time to stop: */
       sanei_thread_kill (s->reader_pid);
       sanei_thread_waitpid (s->reader_pid, &exit_status);
-      s->reader_pid = 0;
+      s->reader_pid = -1;
     }
 
   if (s->fd >= 0)
@@ -489,7 +491,7 @@ do_cancel (Tamarack_Scanner *s)
 static SANE_Status
 get_image_status (Tamarack_Scanner *s)
 {
-  u_int8_t result[12];
+  uint8_t result[12];
   SANE_Status status;
   size_t len;
   int busy;
@@ -597,6 +599,7 @@ init_options (Tamarack_Scanner *s)
 
   s->opt[OPT_NUM_OPTS].title = SANE_TITLE_NUM_OPTIONS;
   s->opt[OPT_NUM_OPTS].desc = SANE_DESC_NUM_OPTIONS;
+  s->opt[OPT_NUM_OPTS].type = SANE_TYPE_INT;
   s->opt[OPT_NUM_OPTS].cap = SANE_CAP_SOFT_DETECT;
   s->val[OPT_NUM_OPTS].w = NUM_OPTIONS;
 
@@ -910,7 +913,7 @@ sane_init (SANE_Int *version_code, SANE_Auth_Callback authorize)
   sanei_thread_init();
 
   if (version_code)
-    *version_code = SANE_VERSION_CODE (V_MAJOR, V_MINOR, 0);
+    *version_code = SANE_VERSION_CODE (SANE_CURRENT_MAJOR, V_MINOR, 0);
 
   fp = sanei_config_open (TAMARACK_CONFIG_FILE);
   if (!fp) {
@@ -1434,7 +1437,7 @@ sane_cancel (SANE_Handle handle)
 {
   Tamarack_Scanner *s = handle;
 
-  if (s->reader_pid > 0)
+  if (s->reader_pid != -1)
     sanei_thread_kill (s->reader_pid);
   s->scanning = SANE_FALSE;
 }
